 package com.huimv.manager.controller;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huimv.manager.entity.*;
import com.huimv.manager.mapper.BaseFarmInfoDao;
import com.huimv.manager.mapper.BasePigpenDao;
import com.huimv.manager.mapper.BasePigpenMapper;
import com.huimv.manager.mapper.BaseRegionDao;
import com.huimv.manager.result.RRException;
import com.huimv.manager.service.BasePigpenService;
import com.huimv.manager.service.BaseRegionService;
import com.huimv.manager.service.EartagDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.huimv.manager.service.CollectorRegisterService;
import com.huimv.manager.result.R;
import com.huimv.manager.utils.PageUtils;


/**
 * 
 *
 * @author yinhao
 * @version ${version}
 * @date 2021-07-24 16:29:33
 */
@RestController
@RequestMapping("manager/collectorregister")
@CrossOrigin
public class CollectorRegisterController {
    @Autowired
    private CollectorRegisterService collectorRegisterService;
    @Autowired
    private EartagDataService eartagDataService;

    @Autowired
    private BasePigpenService basePigpenService;
    /**
     * 列表
     */
    @RequestMapping("/list")
    public R list(@RequestParam Map<String, Object> params){
        PageUtils page = collectorRegisterService.queryPage(params);

        return R.ok().put("page", page);
    }




    /**
     * 信息
     */
    @RequestMapping("/info/{id}")
    public R info(@PathVariable("id") Integer id){
		CollectorRegisterEntity collectorRegister = collectorRegisterService.getById(id);
        BasePigpenEntity pigpen = basePigpenService.getById(collectorRegister.getPigpenId());
        BasePigpenEntity unit = basePigpenService.getById(collectorRegister.getUnitId());
        Map map = new HashMap();
        map.put("acqStatus",collectorRegister.getAcqStatus());
        map.put("canStatus",collectorRegister.getCanStatus());
        map.put("countyCode",collectorRegister.getCountyCode());
        map.put("countyName",collectorRegister.getCountyName());
        map.put("delStatus",collectorRegister.getDelStatus());
        map.put("deviceCode",collectorRegister.getDeviceCode());
        map.put("farmCode",collectorRegister.getFarmCode());
        map.put("farmName",collectorRegister.getFarmName());
        map.put("id",collectorRegister.getId());
        map.put("lastTime",collectorRegister.getLastTime());
        map.put("networkStatus",collectorRegister.getNetworkStatus());
        map.put("pigpenId",collectorRegister.getPigpenId());
        map.put("port",collectorRegister.getPort());
        map.put("registerTime",collectorRegister.getRegisterTime());
        map.put("serverIp",collectorRegister.getServerIp());
        map.put("stage",collectorRegister.getStage());
        map.put("typeF",collectorRegister.getTypeF());
        map.put("unitId",collectorRegister.getUnitId());
        map.put("registeCode",collectorRegister.getRegisteCode());
        map.put("pigpenName",pigpen.getPigpenName());
        map.put("unitName",unit.getPigpenName());
        return R.ok().put("collectorRegister", map);
    }

    @Autowired
    private BaseRegionDao baseRegionDao;

    @Autowired
    private BaseFarmInfoDao baseFarmInfoDao;

    /**
     * 保存
     */
    @RequestMapping("/save")
    public R save(@RequestBody CollectorRegisterEntity collectorRegister){
        QueryWrapper<CollectorRegisterEntity> collectorRegisterEntityQueryWrapper = new QueryWrapper<>();

        String deviceCode = collectorRegister.getDeviceCode();
        collectorRegisterEntityQueryWrapper.eq("device_code",deviceCode);
        int count = collectorRegisterService.count(collectorRegisterEntityQueryWrapper);
        if (count >0){
            return R.error("编号为"+deviceCode+"的设备已存在");
        }
        String countyCode = collectorRegister.getCountyCode();
        QueryWrapper<BaseRegionEntity> baseRegionEntityQueryWrapper = new QueryWrapper<>();
        baseRegionEntityQueryWrapper.eq("region_code",countyCode);
        BaseRegionEntity baseRegionEntity = baseRegionDao.selectOne(baseRegionEntityQueryWrapper);
        QueryWrapper<BaseFarmInfoEntity> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_code",collectorRegister.getFarmCode());
        BaseFarmInfoEntity baseFarmInfoEntity = baseFarmInfoDao.selectOne(wrapper);
        collectorRegister.setCountyName(baseRegionEntity.getRegionName());
        collectorRegister.setFarmName(baseFarmInfoEntity.getFarmName());
        collectorRegisterService.save(collectorRegister);

        return R.ok();
    }

    /**
     * 修改
     */
    @RequestMapping("/update")
    public R update(@RequestBody CollectorRegisterEntity collectorRegister){
		collectorRegisterService.updateById(collectorRegister);

        return R.ok();
    }

    /**
     * 删除
     */
    @RequestMapping("/delete")
    public R delete(@RequestBody Integer[] ids){

        if (ids == null || ids.length == 0){
            return R.error("未选择");
        }
        for (Integer id : ids) {
            CollectorRegisterEntity  collectorRegisterEntity= collectorRegisterService.getById(id);
            String deviceCode = collectorRegisterEntity.getDeviceCode();
            int count = eartagDataService.count(Wrappers.<EartagDataEntity>lambdaQuery().eq(EartagDataEntity::getDeviceCode, deviceCode));
            if (count >0){
                throw new RRException("删除失败,编号为"+deviceCode+"的设备有尚未清理的耳标");
            }
        }
		collectorRegisterService.removeByIds(Arrays.asList(ids));

        return R.ok();
    }

    /**
     * 统计设备状态
     */
    @RequestMapping("/countState")
    public R countState(@RequestParam Map<String, Object> params){
        Map map  = collectorRegisterService.countState(params);

        return R.ok().put("data", map);
    }

    @GetMapping("/everyStatusCollectors")
    public R selectEveryStatusCollectors(@RequestParam("farmCode") String farmCode) {
        return collectorRegisterService.selectEveryStatusCollectors(farmCode);
    }

    @GetMapping("/selectPcEveryStatusCollectors")
    public R selectEveryStatusCollectors() {
        return collectorRegisterService.selectPcEveryStatusCollectors();
    }



}
