package com.huimv.manager.entity.dto;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.huimv.manager.utils.Constant;
import lombok.Data;
import org.apache.commons.lang.StringUtils;
import org.hibernate.validator.constraints.Range;

import javax.validation.constraints.NotBlank;
import java.text.ParseException;
import java.util.Date;

/**
 * <p>
 *
 * </p>
 *
 * @author yinhao
 * @date 2021/7/28 15:06
 */
@Data
public class DataAnalysisDto {

    @NotBlank(message = "区域代码不能为空!")
    private String regionCode;

    private String start;

    private String end;

    @Range(min = 1, max = 3, message = "请传递指定的type参数!")
    private Integer type;

    public String getStart() {
        return start;
    }

    public void setStart(String start) {
        this.start = start;
    }

    public String getEnd() {
        return end;
    }

    public void setEnd(String end) {
        this.end = end;
    }


    public String getDayStart() throws ParseException {
        if (StringUtils.isEmpty(start) && StringUtils.isEmpty(end)) {
            return Constant.SDF_YMD.format(DateUtil.offsetMonth(DateUtil.date(), -1));
        } else if (StringUtils.isEmpty(start) && StringUtils.isNotEmpty(end)) {
            return Constant.SDF_YMD.format(DateUtil.offsetMonth(Constant.SDF_YMD.parse(end), -1));
        } else {
            return start;
        }
    }

    public String getDayEnd() throws ParseException {
        Date nowDate = new Date();
        if (StringUtils.isEmpty(end) && StringUtils.isEmpty(start)) {
            return Constant.SDF_YMD.format(nowDate);
        } else if (StringUtils.isEmpty(end) && StringUtils.isNotEmpty(start)) {
            DateTime time = DateUtil.offsetMonth(Constant.SDF_YMD.parse(start), 1);
            return time.getTime() > nowDate.getTime() ? Constant.SDF_YMD.format(nowDate) : Constant.SDF_YMD.format(time);
        } else {
            return end;
        }
    }

    public String getMonthStart() throws ParseException {
        if (StringUtils.isEmpty(start) && StringUtils.isEmpty(end)) {
            return Constant.SDF_YM.format(DateUtil.offsetMonth(new Date(), -12)) + "-01";
        } else if (StringUtils.isEmpty(start) && StringUtils.isNotEmpty(end)) {
            return Constant.SDF_YM.format(DateUtil.offsetMonth(Constant.SDF_YM.parse(end), -12)) + "-01";
        } else {
            return start + "-01";
        }
    }

    public String getMonthEnd() throws ParseException {
        Date nowDate = new Date();
        if (Constant.SDF_YM.format(nowDate).equals(end) || (StringUtils.isEmpty(end) && StringUtils.isEmpty(start))) {
            return Constant.SDF_YMD.format(nowDate);
        } else if (StringUtils.isEmpty(end) && StringUtils.isNotEmpty(start)) {
            DateTime time = DateUtil.offsetMonth(Constant.SDF_YM.parse(start), 12);
            return time.getTime() > nowDate.getTime() ? Constant.SDF_YMD.format(nowDate) : Constant.SDF_YMD.format(time);
        } else {
            return Constant.SDF_YMD.format(DateUtil.endOfMonth(DateUtil.parse(end, Constant.DATE_FORMAT_YM)));
        }
    }

    public String getYearStart() throws ParseException {
        if (StringUtils.isEmpty(start) && StringUtils.isEmpty(end)) {
            return DateUtil.offsetMonth(new Date(), -60).year() + "-01-01";
        } else if (StringUtils.isEmpty(start) && StringUtils.isNotEmpty(end)) {
            return DateUtil.offsetMonth(Constant.SDF_Y.parse(end), -60).year() + "-01-01";
        } else {
            return start + "-01-01";
        }
    }

    public String getYearEnd() throws ParseException {
        Date nowDate = new Date();
        if (Constant.SDF_Y.format(nowDate).equals(end) || (StringUtils.isEmpty(end) && StringUtils.isEmpty(start))) {
            return Constant.SDF_YMD.format(nowDate);
        } else if (StringUtils.isEmpty(end) && StringUtils.isNotEmpty(start)) {
            DateTime time = DateUtil.offsetMonth(Constant.SDF_Y.parse(start), 60);
            return time.getTime() > nowDate.getTime() ? Constant.SDF_YMD.format(nowDate) : Constant.SDF_YMD.format(time);
        } else {
            return Constant.SDF_YMD.format(DateUtil.endOfYear(DateUtil.parse(end, Constant.DATE_FORMAT_Y)));
        }
    }
}
