package com.huimv.manager.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huimv.manager.entity.vo.CollectorRegisterVo;
import com.huimv.manager.mapper.CollectorRegisterDao;
import com.huimv.manager.result.R;
import com.huimv.manager.result.RRException;
import com.huimv.manager.utils.Constant;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.manager.utils.PageUtils;
import com.huimv.manager.utils.Query;

import com.huimv.manager.entity.CollectorRegisterEntity;
import com.huimv.manager.service.CollectorRegisterService;

/**
 * 
 *
 * @author yinhao
 * @version ${version}
 * @date 2021-07-24 16:29:33
 */
@Service("collectorRegisterService")
public class CollectorRegisterServiceImpl extends ServiceImpl<CollectorRegisterDao, CollectorRegisterEntity> implements CollectorRegisterService {



    @Override
    public PageUtils queryPage(Map<String, Object> params) {
        QueryWrapper<CollectorRegisterEntity> wrapper = new QueryWrapper<>();
        String farmCode = (String)params.get("farmCode");
//        if (StringUtils.isBlank(farmCode)){
//            throw new RRException("牧场未选择");
//        }
        String registerStartTime = (String)params.get("registerStartTime");
        String registerEndTime = (String)params.get("registerEndTime");
        String deviceCode = (String)params.get("deviceCode");
        String countyName = (String)params.get("countyName");
        String farmName = (String)params.get("farmName");
        String acqStatus = (String)params.get("acqStatus");
        String canStatus = (String)params.get("canStatus");
        String networkStatus = (String)params.get("networkStatus");

        wrapper.like(StringUtils.isNotBlank(deviceCode),"register.device_code",deviceCode);
        wrapper.like(StringUtils.isNotBlank(countyName),"county_name",countyName);
        wrapper.like(StringUtils.isNotBlank(farmName),"farm_name",farmName);

        wrapper.eq(StringUtils.isNotBlank(farmCode),"register.farm_code",farmCode);
        if (StringUtils.isNotBlank(acqStatus)){
            wrapper.eq("acq_status",Integer.parseInt(acqStatus));
        }

        if (StringUtils.isNotBlank(canStatus)){
            wrapper.eq("can_status",Integer.parseInt(canStatus));
        }

        if (StringUtils.isNotBlank(networkStatus)){
            wrapper.eq("network_status",Integer.parseInt(networkStatus));
        }
        if(StringUtils.isNotBlank(registerEndTime)){
            registerEndTime = registerStartTime+" 23:59:59";
            wrapper.le("register.register_time",registerEndTime);
        }

        wrapper.ge(StringUtils.isNotBlank(registerEndTime),"register.register_time",registerStartTime);

        wrapper.apply("register.del_status  = 1");

        String sortord = (String)params.get("sortord");
        wrapper.orderByDesc(sortord);
        IPage<CollectorRegisterVo> page = baseMapper.page(new Query<>().getPage(params),wrapper);

        return new PageUtils(page);
    }

    @Override
    public Map countState(Map<String, Object> params) {

        String farmCode = (String)params.get("farmCode");
        if (StringUtils.isBlank(farmCode)){
            throw new RRException("牧场未选择");
        }

        QueryWrapper<CollectorRegisterEntity> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_code",farmCode);

        List<CollectorRegisterEntity> collectorRegisterEntities = baseMapper.selectList(wrapper);
        Map map = new HashMap();
        map.put("allCount",collectorRegisterEntities.size());
        map.put("normalStatus",collectorRegisterEntities.stream().filter(o->o.getNetworkStatus().equals(1) && o.getAcqStatus().equals(1) && o.getCanStatus().equals(1)).count());
        map.put( "abnormalStatus",collectorRegisterEntities.stream().filter(o->o.getAcqStatus().equals(0)).count());
        map.put("cancelStatus",collectorRegisterEntities.stream().filter(o->o.getCanStatus().equals(0)).count());

        return map;
    }

    @Override
    public R selectEveryStatusCollectors(String farmCode) {
        LambdaQueryWrapper<CollectorRegisterEntity> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(CollectorRegisterEntity::getFarmCode, farmCode)
                .eq(CollectorRegisterEntity::getDelStatus, Constant.NORMAL);

        Map<String, Integer> map = new LinkedHashMap<>();

        map.put("total", count(lambdaQuery));

        lambdaQuery.eq(CollectorRegisterEntity::getNetworkStatus, Constant.NORMAL)
                .eq(CollectorRegisterEntity::getAcqStatus, Constant.NORMAL)
                .eq(CollectorRegisterEntity::getCanStatus, Constant.NORMAL);
        map.put("normal", count(lambdaQuery));

        lambdaQuery.clear();
        lambdaQuery.eq(CollectorRegisterEntity::getFarmCode, farmCode)
                .eq(CollectorRegisterEntity::getNetworkStatus, Constant.ABNORMAL)
                .or().eq(CollectorRegisterEntity::getAcqStatus, Constant.ACQ_ABNORMAL)
                .eq(CollectorRegisterEntity::getDelStatus, Constant.NORMAL);
        map.put("abnormal", count(lambdaQuery));

        lambdaQuery.clear();
        lambdaQuery.eq(CollectorRegisterEntity::getFarmCode, farmCode)
                .eq(CollectorRegisterEntity::getCanStatus, Constant.ABNORMAL)
                .eq(CollectorRegisterEntity::getDelStatus, Constant.NORMAL);
        map.put("logout", count(lambdaQuery));

        return R.ok().put("data", map);
    }

    @Override
    public R selectPcEveryStatusCollectors() {
        LambdaQueryWrapper<CollectorRegisterEntity> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(CollectorRegisterEntity::getDelStatus, Constant.NORMAL);

        Map<String, Integer> map = new LinkedHashMap<>();

        map.put("total", count(lambdaQuery));

        lambdaQuery.eq(CollectorRegisterEntity::getNetworkStatus, Constant.NORMAL)
                .eq(CollectorRegisterEntity::getAcqStatus, Constant.NORMAL)
                .eq(CollectorRegisterEntity::getCanStatus, Constant.NORMAL);
        map.put("normal", count(lambdaQuery));

        lambdaQuery.clear();
        lambdaQuery.eq(CollectorRegisterEntity::getNetworkStatus, Constant.ABNORMAL)
                .or().eq(CollectorRegisterEntity::getAcqStatus, Constant.ACQ_ABNORMAL)
                .eq(CollectorRegisterEntity::getDelStatus, Constant.NORMAL);
        map.put("abnormal", count(lambdaQuery));

        lambdaQuery.clear();
        lambdaQuery.eq(CollectorRegisterEntity::getCanStatus, Constant.ABNORMAL)
                .eq(CollectorRegisterEntity::getDelStatus, Constant.NORMAL);
        map.put("logout", count(lambdaQuery));

        return R.ok().put("data", map);
    }
}