package com.huimv.manager.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huimv.manager.entity.BaseFarmInfoEntity;
import com.huimv.manager.entity.vo.DailySowVo;
import com.huimv.manager.entity.vo.DailyVo;
import com.huimv.manager.mapper.BaseFarmInfoDao;
import com.huimv.manager.mapper.DailyTabulateDataDao;
import com.huimv.manager.result.R;
import com.huimv.manager.result.RRException;
import com.huimv.manager.utils.Constant;
import jdk.nashorn.internal.ir.IfNode;
import net.bytebuddy.implementation.bytecode.Throw;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.manager.utils.PageUtils;
import com.huimv.manager.utils.Query;

import com.huimv.manager.entity.DailyTabulateDataEntity;
import com.huimv.manager.service.DailyTabulateDataService;

/**
 * 耳标每日汇总表
 *
 * @author yinhao
 * @version ${version}
 * @date 2021-07-24 16:29:33
 */
@Service("dailyTabulateDataService")
public class DailyTabulateDataServiceImpl extends ServiceImpl<DailyTabulateDataDao, DailyTabulateDataEntity> implements DailyTabulateDataService {
    @Autowired
    private BaseFarmInfoDao baseFarmInfoDao;

    @Override
    public PageUtils queryPage(Map<String, Object> params) {
       String farmCode =(String) params.get("farmCode");
       if (StringUtils.isBlank(farmCode)){
           throw new RRException("牧场未选择");
       }
       String startTime =(String) params.get("startTime");
       String endTime =(String) params.get("endTime");
       String minStockTotal =(String) params.get("minStockTotal");
       String maxStockTotal =(String) params.get("maxStockTotal");
       String minStockNew =(String) params.get("minStockNew");
       String maxStockNew =(String) params.get("maxStockNew");
       String minOfflineNo =(String) params.get("minOfflineNo");
       String maxOfflineNo =(String) params.get("maxOfflineNo");
       String minSowStock =(String) params.get("minSowStock");
       String maxSowStock =(String) params.get("maxSowStock");

        QueryWrapper<DailyTabulateDataEntity> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_code",farmCode);

        wrapper.ge(StringUtils.isNotBlank(startTime),"statistic_date",startTime);
        wrapper.le(StringUtils.isNotBlank(startTime),"statistic_date",endTime);

        wrapper.ge(StringUtils.isNotBlank(minStockTotal),"stock_total",minStockTotal);
        wrapper.le(StringUtils.isNotBlank(maxStockTotal),"stock_total",maxStockTotal);

        wrapper.ge(StringUtils.isNotBlank(minStockNew),"stock_new",minStockNew);
        wrapper.le(StringUtils.isNotBlank(maxStockNew),"stock_new",maxStockNew);

        wrapper.ge(StringUtils.isNotBlank(minOfflineNo),"offline_no",minOfflineNo);
        wrapper.le(StringUtils.isNotBlank(maxOfflineNo),"offline_no",maxOfflineNo);

        wrapper.ge(StringUtils.isNotBlank(minOfflineNo),"sow_stock",minSowStock);
        wrapper.le(StringUtils.isNotBlank(maxOfflineNo),"sow_stock",maxSowStock);
        wrapper.orderByDesc("statistic_date");


        IPage<DailyTabulateDataEntity> page = this.page(
                new Query<DailyTabulateDataEntity>().getPage(params),
                wrapper
        );

        return new PageUtils(page);
    }

    @Override
    public DailyTabulateDataEntity listByFarm(Map<String, Object> params) {
        String date = (String) params.get("date");
        String farmCode = (String) params.get("farmCode");
        if (StringUtils.isBlank(farmCode)){
            throw new RRException("请选择牧场");
        }
        if (StringUtils.isBlank(date) ){
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
            date = simpleDateFormat.format(new Date());
        }
        QueryWrapper<DailyTabulateDataEntity> wrapper = new QueryWrapper<>();
        wrapper.eq("statistic_date",date);
        wrapper.eq("farm_code",farmCode);


        return   baseMapper.selectOne(wrapper);
    }

    @Override
    public List countAllChange(Map<String, Object> params) {
        List<DailyVo> list = new ArrayList<>();

        String farmCode = (String)params.get("farmCode");
        String startDate = (String)params.get("startDate");
        String endDate = (String)params.get("endDate");
        String type = (String)params.get("type");

        if (farmCode == null){
            throw new RRException("请选择牧场");
        }

        QueryWrapper<DailyTabulateDataEntity> wrapper = new QueryWrapper<>();
        wrapper.eq("farm_code",farmCode);
        if (startDate != null && startDate.length() > 0){
            if (startDate.length()==7){
                startDate = startDate+"-01";
            }
            if ( startDate.length()==4){
                startDate = startDate+"-01-01";
            }
        }

       if (endDate != null && endDate.length() > 0){
           if ( endDate.length()==4){
               endDate = endDate+"-12-30";
           }
           if ( endDate.length()==7){
               endDate = endDate+"-30";
           }
       }


        wrapper.ge(StringUtils.isNotBlank(startDate),"statistic_date",startDate);
        wrapper.le(StringUtils.isNotBlank(endDate),"statistic_date",endDate);
        wrapper.groupBy("ydate");
        wrapper.orderByDesc("statistic_date");

        if ("1".equals(type)){
            if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)){
                wrapper.last("limit 7");
            }
            list = baseMapper.selectDay(wrapper);
        }
        if ("2".equals(type)){
            if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)){
                wrapper.last("limit 6");
            }
            list = baseMapper.selectMonth(wrapper);
        }
        if ("3".equals(type)){
            if (StringUtils.isBlank(startDate) || StringUtils.isBlank(endDate)){
                wrapper.last("limit 5");
            }
            list = baseMapper.selectYear(wrapper);
        }

        return list;
    }

    @Override
    public Map countAllsow(Map<String, Object> params) {
        String countyCode = (String)params.get("countyCode");
        String farmCode = (String)params.get("farmCode");

        QueryWrapper<BaseFarmInfoEntity> wrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(countyCode)){
            wrapper.eq("county",countyCode).or().eq("city",countyCode);
        }

        List<String>  farmCodeList = new ArrayList();
        List<BaseFarmInfoEntity> baseFarmInfoEntities = baseFarmInfoDao.selectList(wrapper);

        for (BaseFarmInfoEntity baseFarmInfoEntity : baseFarmInfoEntities) {
            farmCodeList.add( baseFarmInfoEntity.getFarmCode());
        }


        QueryWrapper<DailyTabulateDataEntity> wrapper1 = new QueryWrapper<>();

        wrapper1.in(farmCodeList != null && farmCodeList.size() >0,"farm_code",farmCodeList);
        wrapper1.eq(StringUtils.isNotBlank(farmCode),"farm_code",farmCode);
        wrapper1.eq("statistic_date", DateUtil.beginOfDay(new Date()));
        DailySowVo dailySowVo = baseMapper.counSowAll(wrapper1);


        wrapper1.clear();
        wrapper1.in(farmCodeList != null && farmCodeList.size() >0,"farm_code",farmCodeList);
        wrapper1.eq("statistic_date", DateUtil.offsetDay(DateUtil.beginOfDay(new Date()),-1));
        wrapper1.eq(StringUtils.isNotBlank(farmCode),"farm_code",farmCode);
        DailySowVo yestDayDailySowVo = baseMapper.counSowAll(wrapper1);
        Integer sowTotalStock = dailySowVo.getStockTotal();
        Integer yestdayTotalSowStock = yestDayDailySowVo.getStockTotal();

        Map map = new HashMap();
        map.put("sowStock",dailySowVo.getSowStock());
        if (yestdayTotalSowStock.equals(sowTotalStock)){
            map.put("sowTotalStockValue",0);
            map.put("sowTotalStockType",null);
        }else if(sowTotalStock > yestdayTotalSowStock){
            map.put("sowTotalStockValue",sowTotalStock - yestdayTotalSowStock);
            map.put("sowTotalStockType",true);
        }else {
            map.put("sowTotalStockValue",yestdayTotalSowStock - sowTotalStock);
            map.put("sowTotalStockType",false);
        }

        map.put("stockToal",dailySowVo.getStockTotal());
        map.put("stock1",dailySowVo.getStock1());
        map.put("stock2",dailySowVo.getStock2());
        map.put("stock3",dailySowVo.getStock4());


        return map;
    }


    @Override
    public R selectEverydaySowTotalStockByFarmCode(String farmCode) {
        LambdaQueryWrapper<DailyTabulateDataEntity> lambdaQuery = Wrappers.lambdaQuery();
        Date today = new Date();
        lambdaQuery.eq(DailyTabulateDataEntity::getFarmCode, farmCode)
                .ge(DailyTabulateDataEntity::getStatisticDate, Constant.SDF_YMD.format(DateUtil.offsetDay(today, -7)))
                .le(DailyTabulateDataEntity::getStatisticDate, Constant.SDF_YMD.format(today));

        List<DailyTabulateDataEntity> list = list(lambdaQuery);
        List<String> dateList = new ArrayList<>();
        List<Integer> stockList = new ArrayList<>();
        if (CollUtil.isNotEmpty(list)) {
            for (DailyTabulateDataEntity entity : list) {
                dateList.add(Constant.SDF_MD.format(entity.getStatisticDate()));
                stockList.add(entity.getStockTotal());
            }
        }
        return Objects.requireNonNull(R.ok().put("dateList", dateList)).put("stockList", stockList);
    }
}