package com.huimv.manager.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huimv.manager.entity.BaseRegionEntity;
import com.huimv.manager.entity.dto.DataAnalysisDto;
import com.huimv.manager.entity.vo.DayStockVo;
import com.huimv.manager.entity.vo.SunAreaStockVo;
import com.huimv.manager.mapper.BaseRegionDao;
import com.huimv.manager.mapper.DailyTabulateDataDao;
import com.huimv.manager.result.R;
import com.huimv.manager.result.RRException;
import com.huimv.manager.service.DataAnalysisService;
import com.huimv.manager.utils.AnalysisConstant;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author yinhao
 * @date 2021/7/28 14:17
 */
@Service("dataAnalysisService")
public class DataAnalysisServiceImpl implements DataAnalysisService {

    @Autowired
    private BaseRegionDao baseRegionDao;

    @Autowired
    private DailyTabulateDataDao dailyTabulateDataDao;

    @Override
    public R getDayPigStockByRegionCode(DataAnalysisDto dataAnalysisDto) throws ParseException {

        String regionCode = dataAnalysisDto.getRegionCode();
        BaseRegionEntity baseRegionEntity = selectBaseRegion(regionCode);

        Integer type = dataAnalysisDto.getType();
        String parentCode = baseRegionEntity.getParentCode();
        //省级
        if (parentCode == null) {
            switch (type) {
                case 1:
                    return buildBackData(dataAnalysisDto, AnalysisConstant.PROVINCE_FIELD, AnalysisConstant.DAY_GROUP_FIELD);
                case 2:
                    return buildBackData(dataAnalysisDto, AnalysisConstant.PROVINCE_FIELD, AnalysisConstant.MONTH_GROUP_FIELD);
                case 3:
                    return buildBackData(dataAnalysisDto, AnalysisConstant.PROVINCE_FIELD, AnalysisConstant.YEAR_GROUP_FIELD);
                default:
                    throw new RRException("type参数有误,请检查!");
            }
        } else {
            BaseRegionEntity entity = selectBaseRegion(parentCode);
            //市级
            if (entity.getParentCode() == null) {
                switch (type) {
                    case 1:
                        return buildBackData(dataAnalysisDto, AnalysisConstant.CITY_FIELD, AnalysisConstant.DAY_GROUP_FIELD);
                    case 2:
                        return buildBackData(dataAnalysisDto, AnalysisConstant.CITY_FIELD, AnalysisConstant.MONTH_GROUP_FIELD);
                    case 3:
                        return buildBackData(dataAnalysisDto, AnalysisConstant.CITY_FIELD, AnalysisConstant.YEAR_GROUP_FIELD);
                    default:
                        throw new RRException("type参数有误,请检查!");
                }
                //区县级
            } else {
                switch (type) {
                    case 1:
                        return buildBackData(dataAnalysisDto, AnalysisConstant.COUNTY_FIELD, AnalysisConstant.DAY_GROUP_FIELD);
                    case 2:
                        return buildBackData(dataAnalysisDto, AnalysisConstant.COUNTY_FIELD, AnalysisConstant.MONTH_GROUP_FIELD);
                    case 3:
                        return buildBackData(dataAnalysisDto, AnalysisConstant.COUNTY_FIELD, AnalysisConstant.YEAR_GROUP_FIELD);
                    default:
                        throw new RRException("type参数有误,请检查!");
                }
            }
        }
    }

    private R buildBackData(DataAnalysisDto dataAnalysisDto, String countField, String typeGroupField) throws ParseException {
        QueryWrapper<DayStockVo> query = Wrappers.query();


        String regionCode = dataAnalysisDto.getRegionCode();
        String start = null;
        String end = null;
        int type;

        if (AnalysisConstant.DAY_GROUP_FIELD.equals(typeGroupField)) {
            start = dataAnalysisDto.getDayStart();
            end = dataAnalysisDto.getDayEnd();
            type = 1;
        } else if (AnalysisConstant.MONTH_GROUP_FIELD.equals(typeGroupField)) {
            start = dataAnalysisDto.getMonthStart();
            end = dataAnalysisDto.getMonthEnd();
            type = 2;
        } else {
            start = dataAnalysisDto.getYearStart();
            end = dataAnalysisDto.getYearEnd();
            type = 3;
        }

        query.eq(countField, regionCode)
                .ge("dtd.statistic_date", start)
                .le("dtd.statistic_date", end)
                .groupBy(countField, typeGroupField)
                .orderByAsc("date");

        List<DayStockVo> dayStockVoList = dailyTabulateDataDao.getDayStockByRegion(query, type);
        List<String> dateList = new ArrayList<>();
        List<Integer> stockList = new ArrayList<>();
        fillList(dateList, stockList, dayStockVoList);

//        QueryWrapper<SunAreaStockVo> sunAreaQuery = Wrappers.query();
//        sunAreaQuery.eq(countField, regionCode)
//                .groupBy(regionGroupField, "dtd.statistic_date")
//                .orderByDesc("dtd.statistic_date");
//
//        List<SunAreaStockVo> sunAreaStockVoList = dailyTabulateDataDao.getCityPigStock(sunAreaQuery);
//        List<String> regionList = new ArrayList<>();
//        List<Integer> regionStockList = new ArrayList<>();
//        fillList(regionList, regionStockList, sunAreaStockVoList);

        return R.ok().put("dateList", dateList).put("stockList", stockList);
    }

    private void fillList(List<String> list1, List<Integer> list2, List<?> dbList) {
        for (Object o : dbList) {
            if (o instanceof DayStockVo) {
                list1.add(((DayStockVo) o).getDate());
                list2.add(((DayStockVo) o).getTotal());
            } else if (o instanceof SunAreaStockVo) {
                list1.add(((SunAreaStockVo) o).getRegion());
                list2.add(((SunAreaStockVo) o).getTotal());
            }
        }
    }

    @Override
    public R getChildPigStockByRegionCode(String regionCode) {

        BaseRegionEntity baseRegionEntity = selectBaseRegion(regionCode);
        String parentCode = baseRegionEntity.getParentCode();
        //省级
        if (parentCode == null) {
            return buildChildStockBack(regionCode, AnalysisConstant.PROVINCE_FIELD, AnalysisConstant.CITY_FIELD);
        } else {
            BaseRegionEntity entity = selectBaseRegion(parentCode);
            //市级
            if (entity.getParentCode() == null) {
                return buildChildStockBack(regionCode, AnalysisConstant.CITY_FIELD, AnalysisConstant.COUNTY_FIELD);
                //区县级
            } else {
                return buildChildStockBack(regionCode, AnalysisConstant.COUNTY_FIELD, AnalysisConstant.FARM_FIELD);
            }
        }
    }

    private R buildChildStockBack(String regionCode, String countField, String regionGroupField) {

        QueryWrapper<SunAreaStockVo> sunAreaQuery = Wrappers.query();
        sunAreaQuery.eq(countField, regionCode)
                .groupBy(regionGroupField, "dtd.statistic_date")
                .orderByDesc("dtd.statistic_date");

        List<SunAreaStockVo> sunAreaStockVoList = dailyTabulateDataDao.getCityPigStock(sunAreaQuery,regionGroupField);
        List<String> regionList = new ArrayList<>();
        List<Integer> regionStockList = new ArrayList<>();
        fillList(regionList, regionStockList, sunAreaStockVoList);

        return R.ok().put("regionList", regionList).put("regionStockList", regionStockList);
    }


    private BaseRegionEntity selectBaseRegion(String regionCode) {
        LambdaQueryWrapper<BaseRegionEntity> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(BaseRegionEntity::getRegionCode, regionCode);
        BaseRegionEntity baseRegionEntity = baseRegionDao.selectOne(lambdaQuery);
        if (baseRegionEntity == null) {
            throw new RRException("未查询到该区域信息,请检查!", 10001);
        }
        return baseRegionEntity;
    }


    //   private R provincialDataAnalysis(DataAnalysisDto dataAnalysisDto) {
//        return buildBackData(dataAnalysisDto, AnalysisConstant.PROVINCE_FIELD, AnalysisConstant.CITY_FIELD, AnalysisConstant.DAY_GROUP_FIELD);
//
////        QueryWrapper<DayStockVo> dayStockVoQueryWrapper = buildDayStockQueryWrapper(dataAnalysisDto);
////        dayStockVoQueryWrapper.groupBy("farm.province", "dtd.statistic_date");
////
////        List<DayStockVo> dayStockVoList = dailyTabulateDataDao.getDayStockByRegion(dayStockVoQueryWrapper);
////
////        List<String> dateList = new ArrayList<>();
////        List<Integer> stockList = new ArrayList<>();
////        fillList(dateList, stockList, dayStockVoList);
////
////
////        QueryWrapper<SunAreaStockVo> sunAreaQuery = Wrappers.query();
////        sunAreaQuery.eq("farm.province", dataAnalysisDto.getRegionCode())
////                .groupBy("farm.city", "dtd.statistic_date")
////                .orderByDesc("dtd.statistic_date");
////        List<SunAreaStockVo> sunAreaStockVoList = dailyTabulateDataDao.getCityPigStock(sunAreaQuery);
////        List<String> regionList = new ArrayList<>();
////        List<Integer> regionStockList = new ArrayList<>();
////        fillList(regionList, regionStockList, sunAreaStockVoList);
////
////        return R.ok().put("dateList", dateList)
////                .put("stockList", stockList)
////                .put("regionList", regionList)
////                .put("regionStockList", regionStockList);
//    }
//
//
//    private R municipalDataAnalysis(DataAnalysisDto dataAnalysisDto) {
////        String start = dataAnalysisDto.getDayStart();
////        String end = dataAnalysisDto.getDayEnd();
//
////        QueryWrapper<DayStockVo> query = Wrappers.query();
////        query.eq("farm.city", dataAnalysisDto.getRegionCode())
////                .ge("dtd.statistic_date", start)
////                .le("dtd.statistic_date", end)
////                .groupBy("farm.city", "dtd.statistic_date")
////                .orderByAsc("date");
////
////        List<DayStockVo> dayStockVoList = dailyTabulateDataDao.getMunicipalDayStock(query);
//
//        return buildBackData(dataAnalysisDto, AnalysisConstant.CITY_FIELD, AnalysisConstant.COUNTY_FIELD, AnalysisConstant.DAY_GROUP_FIELD);
////        QueryWrapper<DayStockVo> dayStockVoQueryWrapper = buildDayStockQueryWrapper(dataAnalysisDto);
////        dayStockVoQueryWrapper.groupBy("farm.city", "dtd.statistic_date");
////        List<DayStockVo> dayStockVoList = dailyTabulateDataDao.getDayStockByRegion(dayStockVoQueryWrapper);
////
////
////        List<String> dateList = new ArrayList<>();
////        List<Integer> stockList = new ArrayList<>();
////        fillList(dateList, stockList, dayStockVoList);
////
////        QueryWrapper<SunAreaStockVo> sunAreaQuery = Wrappers.query();
////        sunAreaQuery.eq("farm.city", dataAnalysisDto.getRegionCode())
////                .groupBy("farm.county", "dtd.statistic_date")
////                .orderByDesc("dtd.statistic_date");
////        List<SunAreaStockVo> sunAreaStockVoList = dailyTabulateDataDao.getCityPigStock(sunAreaQuery);
////        List<String> regionList = new ArrayList<>();
////        List<Integer> regionStockList = new ArrayList<>();
////        fillList(regionList, regionStockList, sunAreaStockVoList);
////
////        return R.ok().put("dateList", dateList)
////                .put("stockList", stockList)
////                .put("regionList", regionList)
////                .put("regionStockList", regionStockList);
//
//    }

//        private R countyDataAnalysis(DataAnalysisDto dataAnalysisDto) {
//            return buildBackData(dataAnalysisDto, AnalysisConstant.COUNTY_FIELD, AnalysisConstant.FARM_FIELD, AnalysisConstant.DAY_GROUP_FIELD);
//        }
}
