package com.huimv.manager.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.huimv.manager.entity.*;
import com.huimv.manager.entity.vo.EartagDataCountVo;
import com.huimv.manager.mapper.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.manager.utils.PageUtils;
import com.huimv.manager.utils.Query;

import com.huimv.manager.service.EartagDataService;

/**
 * @author yinhao
 * @version ${version}
 * @date 2021-07-24 16:29:33
 */
@Service("eartagDataService")
public class EartagDataServiceImpl extends ServiceImpl<EartagDataDao, EartagDataEntity> implements EartagDataService {


    @Override
    public PageUtils queryPage(Map<String, Object> params) {


        String time = (String) params.get("time");//采集时间
        //String farmName = (String) params.get("farmName");//牧场名称
        String eartagNo = (String) params.get("eartagNo");//耳标号
        String dayAge = (String) params.get("dayAge");//耳标号
        String earTemp = (String) params.get("earTemp");//耳根温度
        String envTemp = (String) params.get("envTemp");//环境温度
        String bat = (String) params.get("bat");//电量
        String rssi = (String) params.get("rssi");//信号强度
        String pigpenName = (String) params.get("pigpenName");
        String deviceCode = (String) params.get("deviceCode");

        //TODO 多条件查询
        QueryWrapper<EartagDataEntity> wrapper = new QueryWrapper<>();


        IPage<EartagDataEntity> page = this.page(new Query<EartagDataEntity>().getPage(params), wrapper);
        return new PageUtils(page);
    }

    @Override
    public Map countByTime(Map<String, Object> params) {
        String startTime = (String) params.get("startTime");
        String endTime = (String) params.get("endTime");
        String deviceCode = (String) params.get("deviceCode");


        QueryWrapper<EartagDataEntity> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(deviceCode),"device_code", deviceCode);
        wrapper.ge(StringUtils.isNotBlank(startTime), "time", startTime);
        wrapper.le(StringUtils.isNotBlank(endTime), "time", endTime);
        wrapper.groupBy("count_time");
        List<EartagDataCountVo> eartagDataCountVos = baseMapper.countByTime(wrapper);

        List timeList = new ArrayList();
        List envTempList = new ArrayList();
        List count = new ArrayList();
        for (EartagDataCountVo eartagDataCountVo : eartagDataCountVos) {
           timeList.add(eartagDataCountVo.getCountTime());
           envTempList.add(eartagDataCountVo.getEnvTemp());
           count.add(eartagDataCountVo.getUploadCount());
        }
        Map map = new HashMap();
        map.put("time",timeList);
        map.put("envTemp",envTempList);
        map.put("count",count);


        return map;
    }


    @Autowired
    private DailyTabulateDataDao dailyTabulateDataDao;

    @Autowired
    private EartagOffineDao eartagOffineDao;

    @Autowired
    private BaseFarmInfoDao baseFarmInfoDao;

    @Autowired
    private FarmStockDao farmStockDao;

    @Autowired
    private StockOutDao stockOutDao;

    @Override
    public void updateDailyData() {
        QueryWrapper<BaseFarmInfoEntity> baseFarmInfoWrapper = new QueryWrapper<>();
        List<BaseFarmInfoEntity> baseFarmInfoEntities = baseFarmInfoDao.selectList(baseFarmInfoWrapper);
        for (BaseFarmInfoEntity baseFarmInfoEntity : baseFarmInfoEntities) {
            String farmCode = baseFarmInfoEntity.getFarmCode();
            String farmName = baseFarmInfoEntity.getFarmName();
            String county = baseFarmInfoEntity.getCounty();
            String city = baseFarmInfoEntity.getCity();


            //查询当日牧场下耳标数据
            Date now = new Date();
            QueryWrapper<EartagDataEntity> wrapper = new QueryWrapper<>();
            wrapper.eq("farm_code", farmCode);
            wrapper.ge("time", DateUtil.beginOfDay(now));
            wrapper.groupBy("eartag_no");
            List<EartagDataEntity> eartagDataEntities = baseMapper.selectList(wrapper);

            //过滤
            Set<String> nowSet = new HashSet();
            for (EartagDataEntity eartagDataEntity : eartagDataEntities) {
                nowSet.add(eartagDataEntity.getEartagNo());
            }

            //查询昨日牧场下耳标数据
            wrapper.clear();
            wrapper.eq("farm_code", farmCode);
            wrapper.le("time", DateUtil.beginOfDay(now));
            wrapper.ge("time", DateUtil.offsetDay(DateUtil.beginOfDay(now), -1));
            wrapper.groupBy("eartag_no");
            List<EartagDataEntity> yesterdayEartagDataEntities = baseMapper.selectList(wrapper);

            //过滤
            Set<String> yesterdaySet = new HashSet();
            for (EartagDataEntity eartagDataEntity : yesterdayEartagDataEntities) {
                yesterdaySet.add(eartagDataEntity.getEartagNo());
            }
            Set<String> set = ObjectUtil.cloneByStream(nowSet);

            //当日母猪总存栏
            int allSow = nowSet.size();

            //得到新增
            nowSet.removeAll(yesterdaySet);
            int newSow = nowSet.size();
            //得到离线
            yesterdaySet.removeAll(set);
            int offSow = yesterdaySet.size();

            //添加离线表
            for (String eartagNo : yesterdaySet) {
                wrapper.clear();
                wrapper.eq("eartag_no", eartagNo);
                wrapper.eq("farm_code", farmCode);
                wrapper.orderByDesc("time");
                EartagDataEntity dataEntity = baseMapper.selectList(wrapper).get(0);

                EartagOffineEntity eartagOffineEntity = new EartagOffineEntity();
                eartagOffineEntity.setEartagNo(eartagNo);
                eartagOffineEntity.setFarmCode(farmCode);
                eartagOffineEntity.setDeviceCode(dataEntity.getDeviceCode());
                eartagOffineEntity.setLastDate(dataEntity.getTime());
                eartagOffineEntity.setRegisterDate(now);
                eartagOffineDao.insert(eartagOffineEntity);
            }


            //计算日龄
            Integer ratio1 = 0, ratio2 = 0, ratio3 = 0, ratio4 = 0;
            //1:配怀存栏,2:分娩存栏,3:后备存栏,4:空怀母猪,5:公猪存栏,
            Integer stock1 = 0, stock2 = 0, stock3 = 0, stock4 = 0, stock5 = 0;
            for (EartagDataEntity eartagDataEntity : eartagDataEntities) {
                Integer dayAge = eartagDataEntity.getDayAge2();
                if (dayAge != null) {
                    if (dayAge > 250 && dayAge < 1000) {
                        ratio1++;
                    }

                    if (dayAge >= 1000 && dayAge < 1700) {
                        ratio2++;
                    }

                    if (dayAge >= 1700 && dayAge < 2400) {
                        ratio3++;
                    }

                    if (dayAge >= 2400) {
                        ratio4++;
                    }

                    Integer stage = eartagDataEntity.getStage();
                    if (stage == 1) {
                        stock1++;
                    }
                    if (stage == 2) {
                        stock2++;
                    }
                    if (stage == 3) {
                        stock3++;
                    }
                    if (stage == 4) {
                        stock4++;
                    }
                    if (stage == 5) {
                        stock5++;
                    }
                }
            }
            String ratioStr1 = CalculateUtil(BigDecimal.valueOf(ratio1), BigDecimal.valueOf(allSow));
            String ratioStr2 = CalculateUtil(BigDecimal.valueOf(ratio2), BigDecimal.valueOf(allSow));
            String ratioStr3 = CalculateUtil(BigDecimal.valueOf(ratio3), BigDecimal.valueOf(allSow));
            String ratioStr4 = CalculateUtil(BigDecimal.valueOf(ratio4), BigDecimal.valueOf(allSow));


            //新数据
            QueryWrapper<DailyTabulateDataEntity> dailyWrapper = new QueryWrapper<>();
            dailyWrapper.eq("statistic_date", DateUtil.beginOfDay(now));
            dailyWrapper.eq("farm_code", farmCode);
            DailyTabulateDataEntity dailyTabulateDataEntity = dailyTabulateDataDao.selectOne(dailyWrapper);
            if (dailyTabulateDataEntity == null) {
                dailyTabulateDataEntity = new DailyTabulateDataEntity();
            }
            dailyTabulateDataEntity.setFarmCode(farmCode);
            dailyTabulateDataEntity.setStockTotal(allSow);
            dailyTabulateDataEntity.setStatisticDate(now);
            dailyTabulateDataEntity.setStockNew(newSow);
            dailyTabulateDataEntity.setOfflineNo(offSow);
            dailyTabulateDataEntity.setRatio1(ratio1 + " (" + ratioStr1 + ")");
            dailyTabulateDataEntity.setRatio2(ratio2 + " (" + ratioStr2 + ")");
            dailyTabulateDataEntity.setRatio3(ratio3 + " (" + ratioStr3 + ")");
            dailyTabulateDataEntity.setRatio4(ratio4 + " (" + ratioStr4 + ")");
            dailyTabulateDataEntity.setStock1(stock1);
            dailyTabulateDataEntity.setStock2(stock2);
            dailyTabulateDataEntity.setStock3(stock3);
            dailyTabulateDataEntity.setStock4(stock4);
            dailyTabulateDataEntity.setStock5(stock5);
            dailyTabulateDataEntity.setSowStock(stock1 + stock2 + stock3);

            if (dailyTabulateDataEntity.getId() == null) {
                dailyTabulateDataDao.insert(dailyTabulateDataEntity);
            } else {
                dailyTabulateDataDao.updateById(dailyTabulateDataEntity);
            }


            //牧场存栏统计
            QueryWrapper<FarmStockEntity> farmStockEQueryWrapper = new QueryWrapper<>();
            farmStockEQueryWrapper.eq("year", DateUtil.year(now));
            farmStockEQueryWrapper.eq("month", DateUtil.month(now)+1);
            farmStockEQueryWrapper.eq("farm_code", farmCode);
            FarmStockEntity farmStockEntity = farmStockDao.selectOne(farmStockEQueryWrapper);
            if (farmStockEntity == null) {
                farmStockEntity = new FarmStockEntity();
            }

            farmStockEntity.setFarmCode(farmCode);
            farmStockEntity.setFarmName(farmName);
            farmStockEntity.setSowTotalStock(allSow);
            farmStockEntity.setEartagNew(newSow);
            farmStockEntity.setEartagOffone(offSow);
            farmStockEntity.setSowStock(stock1 + stock2 + stock3);
            farmStockEntity.setStock1(stock1);
            farmStockEntity.setStock2(stock2);
            farmStockEntity.setStock3(stock3);
            farmStockEntity.setStock4(stock4);
            farmStockEntity.setStock5(stock5);
            farmStockEntity.setYear(DateUtil.year(now));
            farmStockEntity.setMonth(DateUtil.month(now)+1);
            farmStockEntity.setCityCode(city);
            farmStockEntity.setCountyCode(county);
            if (farmStockEntity.getId() == null) {
                farmStockDao.insert(farmStockEntity);
            } else {
                farmStockDao.updateById(farmStockEntity);
            }

            //stockOut更新
            QueryWrapper<StockOutEntity> stockOutEntityQueryWrapper = new QueryWrapper<>();
            stockOutEntityQueryWrapper.eq("year", DateUtil.year(now));
            stockOutEntityQueryWrapper.eq("region_code", farmCode);
            StockOutEntity stockOutEntity = stockOutDao.selectOne(stockOutEntityQueryWrapper);
            if (stockOutEntity == null) {
                stockOutEntity = new StockOutEntity();
            }
            stockOutEntity.setRegionCode(farmCode);
            stockOutEntity.setRegionName(farmName);
            stockOutEntity.setSowTotalStock(allSow);
            stockOutEntity.setYear(DateUtil.year(now));

            if (stockOutEntity.getId() == null) {
                stockOutDao.insert(stockOutEntity);
            } else {
                stockOutDao.updateById(stockOutEntity);
            }


        }


    }

    //计算百分比
    public String CalculateUtil(BigDecimal a, BigDecimal b) {
        String percent =
                b == null ? "-" :
                        b.compareTo(new BigDecimal(0)) == 0 ? "-" :
                                a == null ? "0.00%" :
                                        a.multiply(new BigDecimal(100)).divide(b, 2, BigDecimal.ROUND_HALF_UP) + "%";
        return percent;
    }

    @Override
    public Map countByDay(Map<String, Object> params) {

        String startTime = (String) params.get("startTime");
        String endTime = (String) params.get("endTime");
        String deviceCode = (String) params.get("countByDay");


        QueryWrapper<EartagDataEntity> wrapper = new QueryWrapper<>();
        wrapper.eq(StringUtils.isNotBlank(deviceCode),"device_code", deviceCode);
        wrapper.ge(StringUtils.isNotBlank(startTime), "time", startTime);
        wrapper.le(StringUtils.isNotBlank(endTime), "time", endTime);
        wrapper.groupBy("count_time");
        List<EartagDataCountVo> eartagDataCountVos = baseMapper.countByDay(wrapper);

        List timeList = new ArrayList();
        List envTempList = new ArrayList();
        List count = new ArrayList();
        for (EartagDataCountVo eartagDataCountVo : eartagDataCountVos) {
            timeList.add(eartagDataCountVo.getCountTime());
            envTempList.add(eartagDataCountVo.getEnvTemp());
            count.add(eartagDataCountVo.getUploadCount());
        }
        Map map = new HashMap();
        map.put("time",timeList);
        map.put("envTemp",envTempList);
        map.put("count",count);


        return map;
    }
}