package com.huimv.transmission.controller;

import com.huimv.transmission.entity.form.*;
import com.huimv.transmission.result.R;
import com.huimv.transmission.result.RRException;
import com.huimv.transmission.service.DailyTabulateDataService;
import com.huimv.transmission.service.DataPushService;
import com.huimv.transmission.service.EartagDataService;
import com.huimv.transmission.service.FarmEartagChangeService;
import com.huimv.transmission.utils.PageUtils;
import com.huimv.transmission.utils.TokenUtil;
import org.apache.commons.codec.binary.Hex;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Calendar;
import java.util.Date;


/**
 * <p>
 * 电信智慧养殖云平台接口
 * </p>
 *
 * @author yinhao
 * @date 2021/7/26 10:19
 */
@CrossOrigin
@RestController
@RequestMapping("/pig")
public class PigDataProcessingController {

    @Autowired
    private DataPushService dataPushService;

    @Autowired
    private EartagDataService eartagDataService;

    @Autowired
    private DailyTabulateDataService dailyTabulateDataService;

    @Autowired
    private FarmEartagChangeService farmEartagChangeService;


    /**
     * 1. 申请 AccessToken 令牌
     * @param tokenForm
     * @return
     * @throws NoSuchAlgorithmException
     */
    @GetMapping("/getAccessToken")
    public R getAccessToken(@Validated TokenForm tokenForm) throws NoSuchAlgorithmException {

        Long timestamp = tokenForm.getTimestamp();
        try {
            new Date(timestamp);
        } catch (Exception e) {
            throw new RRException("时间戳有误!", 10002);
        }
        MessageDigest md = MessageDigest.getInstance("MD5");
        String data = tokenForm.getRandom() + ";" + timestamp;
        String oneSign = Hex.encodeHexString(md.digest(data.getBytes(StandardCharsets.UTF_8)));
        oneSign = oneSign + "zhejiangxumu";
        String twoSign = Hex.encodeHexString(md.digest(oneSign.getBytes(StandardCharsets.UTF_8)));
        System.out.println(twoSign);

        if (!tokenForm.getSign().equals(twoSign)) {
            throw new RRException("签名验证失败!", 10003);
        }

        String token = TokenUtil.sign(null);
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.HOUR_OF_DAY, 24);
        Date expireTime = calendar.getTime();
        return R.ok().put("accessToken", token).put("expireTime", expireTime);
    }

    /**
     * 2. 生猪存栏数读取接口
     * @param pigStockForm
     * @return
     */
    @GetMapping("/getPigStock")
    public R getPigStock(PigStockForm pigStockForm) {
        PageUtils page = dailyTabulateDataService.getPigStock(pigStockForm);
        return R.ok().put("resultData", page);
    }

    /**
     * 3. 新增生猪耳标号读取接口
     * @param newAndOffLineForm
     * @return
     */
    @GetMapping("/getNewEartagNo")
    public R getNewPigStock(NewAndOffLineForm newAndOffLineForm) {
        PageUtils page = farmEartagChangeService.getChangePigStock(newAndOffLineForm);
        return R.ok().put("resultData", page);
    }

    /**
     * 4. 离线生猪耳标号读取接口
     * @param newAndOffLineForm
     * @return
     */
    @GetMapping("/getEliminateEartagNo")
    public R getOffLinePigStock(NewAndOffLineForm newAndOffLineForm) {
        PageUtils page = farmEartagChangeService.getOffLinePigStock(newAndOffLineForm);
        return R.ok().put("resultData", page);
    }


    /**
     * 5. 智能耳标接口
     * @param eartagForm
     * @return
     */
    @GetMapping("/getEartag")
    public R getEartag(@Validated EartagForm eartagForm) {
        PageUtils page = eartagDataService.getEartag(eartagForm);
        return R.ok().put("resultData", page);
    }

    /**
     * 6. 耳标明细数据读取接口
     * @param eartagDetailForm
     * @return
     */
    @GetMapping("/getEartagDetails")
    public R getEartagDetails(@Validated EartagDetailForm eartagDetailForm) {
        PageUtils page = eartagDataService.getEartagDetails(eartagDetailForm);
        return R.ok().put("resultData", page);
    }

    /**
     * 7. 每日存栏统计读取接口
     * @param baseForm
     * @return
     */
    @GetMapping("/getPigStockByDay")
    public R getPigStockByDay(@Validated BaseForm baseForm) {
        PageUtils page = dailyTabulateDataService.getPigStockByDay(baseForm);
        return R.ok().put("resultData", page);
    }


    /**
     * 测试接收推送数据接口
     * @param body
     */
    @PostMapping("/recive")
    public void receiveData(@RequestBody String body) {
        System.out.println(body);
    }
}
