package com.huimv.transmission.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.huimv.transmission.entity.BaseFarmInfoEntity;
import com.huimv.transmission.entity.DailyTabulateDataEntity;
import com.huimv.transmission.entity.form.BaseForm;
import com.huimv.transmission.entity.form.PigStockForm;
import com.huimv.transmission.entity.vo.DailyTabulateDataVo;
import com.huimv.transmission.entity.vo.DayPigStockVo;
import com.huimv.transmission.mapper.BaseFarmInfoDao;
import com.huimv.transmission.mapper.DailyTabulateDataDao;
import com.huimv.transmission.result.RRException;
import com.huimv.transmission.service.DailyTabulateDataService;
import com.huimv.transmission.utils.PageUtils;
import com.huimv.transmission.utils.Query;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * 耳标每日汇总表
 *
 * @author yinhao
 * @version ${version}
 * @date 2021-07-24 16:29:33
 */
@Service("dailyTabulateDataService")
public class DailyTabulateDataServiceImpl extends ServiceImpl<DailyTabulateDataDao, DailyTabulateDataEntity> implements DailyTabulateDataService {

    @Autowired
    private BaseFarmInfoDao baseFarmInfoDao;

    @Override
    public PageUtils queryPage(Map<String, Object> params) {
        IPage<DailyTabulateDataEntity> page = this.page(
                new Query<DailyTabulateDataEntity>().getPage(params),
                new QueryWrapper<>()
        );

        return new PageUtils(page);
    }

    @Override
    public IPage<DailyTabulateDataVo> getPigStockPage(PigStockForm pigStockForm) {
        IPage<DailyTabulateDataVo> iPage = new Page<>(pigStockForm.getPageNo(), pigStockForm.getPageSize());
        LambdaQueryWrapper<DailyTabulateDataEntity> lambdaQuery = Wrappers.lambdaQuery();
        String yesterday = DateUtil.format(new Date(), "yyyy-MM-dd");
        lambdaQuery.eq(DailyTabulateDataEntity::getStatisticDate, yesterday);

        return baseMapper.getPigStockPage(iPage, lambdaQuery);
    }

    @Override
    public PageUtils getPigStockByDay(BaseForm baseForm) {
        String farmID = baseForm.getFarmID();
        if (StringUtils.isNotEmpty(farmID)) {
            EartagDataServiceImpl.checkFarmCode(farmID, baseFarmInfoDao);
        }

        LambdaQueryWrapper<DailyTabulateDataEntity> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(StringUtils.isNotEmpty(farmID), DailyTabulateDataEntity::getFarmCode, farmID)
                .ge(baseForm.getStartDate() != null, DailyTabulateDataEntity::getStatisticDate, baseForm.getStartDate())
                .le(baseForm.getEndDate() != null, DailyTabulateDataEntity::getStatisticDate, baseForm.getEndDate())
                .orderByDesc(DailyTabulateDataEntity::getStatisticDate);

        IPage<DayPigStockVo> iPage = new Page<>(baseForm.getPageNo(), baseForm.getPageSize());
        IPage<DayPigStockVo> page = baseMapper.getPigStockByDay(iPage, lambdaQuery);

        return new PageUtils(page);
    }

    @Override
    public PageUtils getPigStock(PigStockForm pigStockForm) {
        LambdaQueryWrapper<DailyTabulateDataEntity> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(StringUtils.isNotEmpty(pigStockForm.getFarmID()), DailyTabulateDataEntity::getFarmCode, pigStockForm.getFarmID())
                .ge(pigStockForm.getStartDate() != null, DailyTabulateDataEntity::getStatisticDate, pigStockForm.getStartDate())
                .le(pigStockForm.getEndDate() != null, DailyTabulateDataEntity::getStatisticDate, pigStockForm.getEndDate());

        IPage<DailyTabulateDataVo> iPage = new Page<>(pigStockForm.getPageNo(), pigStockForm.getPageSize());
        IPage<DailyTabulateDataVo> page = baseMapper.getPigStockPage(iPage, lambdaQuery);
        return new PageUtils(page);

    }
}