package com.huimv.transmission.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.transmission.entity.BaseFarmInfoEntity;
import com.huimv.transmission.entity.EartagDataEntity;
import com.huimv.transmission.entity.NewAndOffLineEartagEntity;
import com.huimv.transmission.entity.form.EartagDetailForm;
import com.huimv.transmission.entity.form.EartagForm;
import com.huimv.transmission.entity.vo.EartagDataVo;
import com.huimv.transmission.entity.vo.EartagDetailVo;
import com.huimv.transmission.mapper.BaseFarmInfoDao;
import com.huimv.transmission.mapper.EartagDataDao;
import com.huimv.transmission.mapper.FarmEartagChangeDao;
import com.huimv.transmission.result.RRException;
import com.huimv.transmission.service.EartagDataService;
import com.huimv.transmission.utils.PageUtils;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * @author yinhao
 * @version ${version}
 * @date 2021-07-24 16:29:33
 */
@Service("eartagDataService")
public class EartagDataServiceImpl extends ServiceImpl<EartagDataDao, EartagDataEntity> implements EartagDataService {

    @Autowired
    private FarmEartagChangeDao farmEartagChangeDao;

    @Autowired
    private BaseFarmInfoDao baseFarmInfoDao;


    @Override
    @Transactional(rollbackFor = Throwable.class)
    public void handleNewAndOffLineEartag() {
        List<NewAndOffLineEartagEntity> list = baseMapper.selectNowDayChangeEartagList();
        HashSet<String> todaySet = new HashSet<>();
        HashSet<String> todaySetTemp = new HashSet<>();
        HashSet<String> beforeSet = new HashSet<>();

        if (CollUtil.isEmpty(list)) {
            farmEartagChangeDao.truncateTable();
            return;
        }

        for (NewAndOffLineEartagEntity entity : list) {
            String farmCode = entity.getFarmCode();
            String todayEartags = entity.getTodayEartags();
            if (StringUtils.isNotEmpty(todayEartags)) {
                String[] split = todayEartags.split(",");
                todaySet.addAll(Arrays.asList(split));
                todaySetTemp.addAll(Arrays.asList(split));
            }

            String beforeEartags = entity.getBeforeEartags();
            if (StringUtils.isNotEmpty(beforeEartags)) {
                String[] split = beforeEartags.split(",");
                beforeSet.addAll(Arrays.asList(split));
            }

            todaySet.removeAll(beforeSet);
            beforeSet.removeAll(todaySetTemp);

            farmEartagChangeDao.truncateTable();

            if (CollUtil.isNotEmpty(todaySet)) {
                farmEartagChangeDao.batchInsertNewEartags(farmCode, 1, todaySet);
            }
            if (CollUtil.isNotEmpty(beforeSet)) {
                farmEartagChangeDao.batchInsertNewEartags(farmCode, 2, beforeSet);
            }

            todaySet.clear();
            beforeSet.clear();
            todaySetTemp.clear();
        }
    }

    @Override
    public PageUtils getEartag(EartagForm eartagForm) {

        String farmID = eartagForm.getFarmID();
        if (StringUtils.isNotEmpty(farmID)) {
            checkFarm(farmID);
        }

        String eartag = eartagForm.getEartag();
        if (StringUtils.isNotEmpty(eartag)) {
            int count = count(Wrappers.<EartagDataEntity>lambdaQuery().eq(EartagDataEntity::getEartagNo, eartag).last("limit 1"));
            if (count <= 0) {
                throw new RRException("耳标信息不存在,请检查!", 10001);
            }
        }

        IPage<EartagDataVo> ipage = new Page<>(eartagForm.getPageNo(), eartagForm.getPageSize());

        LambdaQueryWrapper<EartagDataEntity> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.ge(eartagForm.getStartDate() != null, EartagDataEntity::getTime, eartagForm.getStartDate())
                .le(eartagForm.getEndDate() != null, EartagDataEntity::getTime, eartagForm.getEndDate())
                .eq(farmID != null, EartagDataEntity::getFarmCode, farmID)
                .like(eartag != null, EartagDataEntity::getEartagNo, eartag)
                .orderByDesc(EartagDataEntity::getId);

        IPage<EartagDataVo> page = baseMapper.selectByConditions(ipage, lambdaQuery);
        return new PageUtils(page);
    }

    @Override
    public PageUtils getEartagDetails(EartagDetailForm eartagDetailForm) {

        String farmID = eartagDetailForm.getFarmID();
        if (StringUtils.isNotEmpty(farmID)) {
            checkFarm(farmID);
        }

        QueryWrapper<EartagDetailVo> query = Wrappers.query();
        query.eq("eartag.farm_code", farmID)
                .ge(eartagDetailForm.getStartDate() != null, "eartag.time", eartagDetailForm.getStartDate())
                .le(eartagDetailForm.getEndDate() != null, "eartag.time", eartagDetailForm.getEndDate())
                .eq(eartagDetailForm.getStage() != null, "eartag.stage", eartagDetailForm.getStage());

        IPage<EartagDetailVo> iPage = new Page<>(eartagDetailForm.getPageNo(), eartagDetailForm.getPageSize());
        IPage<EartagDetailVo> page = baseMapper.selectEartagDetails(iPage, query);

        return new PageUtils(page);
    }


    private void checkFarm(String farmID) {
        checkFarmCode(farmID, baseFarmInfoDao);
    }

    static void checkFarmCode(String farmID, BaseFarmInfoDao baseFarmInfoDao) {
        Integer count = baseFarmInfoDao.selectCount(Wrappers.<BaseFarmInfoEntity>lambdaQuery().eq(BaseFarmInfoEntity::getFarmCode, farmID).last("limit 1"));
        if (count <= 0) {
            throw new RRException("farmID不存在,请检查!", 10001);
        }
    }
}