package com.huimv.transmission.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.transmission.entity.EartagDataEntity;
import com.huimv.transmission.entity.FarmEartagChangeEntity;
import com.huimv.transmission.entity.NewAndOffLineEartagEntity;
import com.huimv.transmission.entity.form.NewAndOffLineForm;
import com.huimv.transmission.entity.form.PigStockForm;
import com.huimv.transmission.entity.vo.ChangeEartagDataVo;
import com.huimv.transmission.entity.vo.OffLineEartagsVo;
import com.huimv.transmission.mapper.BaseFarmInfoDao;
import com.huimv.transmission.mapper.EartagDataDao;
import com.huimv.transmission.mapper.FarmEartagChangeDao;
import com.huimv.transmission.service.EartagDataService;
import com.huimv.transmission.service.FarmEartagChangeService;
import com.huimv.transmission.utils.PageUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *
 * </p>
 *
 * @author yinhao
 * @date 2021/7/26 17:48
 */
@Service("farmEartagChangeService")
public class FarmEartagChangeServiceImpl extends ServiceImpl<FarmEartagChangeDao, FarmEartagChangeEntity> implements FarmEartagChangeService {

    @Autowired
    private BaseFarmInfoDao baseFarmInfoDao;

    @Autowired
    private EartagDataDao eartagDataDao;

    @Override
    public IPage<ChangeEartagDataVo> getPigStockByChangeType(Integer currPage, Integer pageSize, Integer type) {
        IPage<ChangeEartagDataVo> iPage = new Page<>(currPage, pageSize);
        LambdaQueryWrapper<FarmEartagChangeEntity> lambdaQuery = Wrappers.lambdaQuery();
        lambdaQuery.eq(FarmEartagChangeEntity::getChangeType, type);
        return baseMapper.getPigStockByChangeType(iPage, lambdaQuery);
    }

    @Override
    public PageUtils getChangePigStock(NewAndOffLineForm newAndOffLineForm) {

        IPage<NewAndOffLineEartagEntity> page = selectNewAndOffLineEartags(newAndOffLineForm);
        List<NewAndOffLineEartagEntity> records = page.getRecords();

        List<ChangeEartagDataVo> collect = records.stream().map(entity -> {
            ChangeEartagDataVo changeEartagDataVo = new ChangeEartagDataVo();
            String beforeEartags = entity.getBeforeEartags();
            String todayEartags = entity.getTodayEartags();
            List<String> list = buildList(todayEartags, beforeEartags);
            changeEartagDataVo.setFarmID(entity.getFarmCode());
            changeEartagDataVo.setNewEartagNo(list);
            return changeEartagDataVo;
        }).collect(Collectors.toList());

        IPage<ChangeEartagDataVo> newEartagPage = new Page<>();
        buildPage(page, newEartagPage);
        newEartagPage.setRecords(collect);

        return new PageUtils(newEartagPage);
    }

    @Override
    public PageUtils getOffLinePigStock(NewAndOffLineForm newAndOffLineForm) {

        IPage<NewAndOffLineEartagEntity> page = selectNewAndOffLineEartags(newAndOffLineForm);
        List<NewAndOffLineEartagEntity> records = page.getRecords();

        List<OffLineEartagsVo> collect = records.stream().map(entity -> {
            OffLineEartagsVo offLineEartagsVo = new OffLineEartagsVo();
            String todayEartags = entity.getTodayEartags();
            String beforeEartags = entity.getBeforeEartags();
            List<String> list = buildList(beforeEartags, todayEartags);
            offLineEartagsVo.setFarmID(entity.getFarmCode());
            offLineEartagsVo.setEliminateEartagNo(list);
            return offLineEartagsVo;
        }).collect(Collectors.toList());

        IPage<OffLineEartagsVo> eliminateEartagPage = new Page<>();
        buildPage(page, eliminateEartagPage);
        eliminateEartagPage.setRecords(collect);
        return new PageUtils(eliminateEartagPage);
    }

    private IPage<NewAndOffLineEartagEntity> selectNewAndOffLineEartags(NewAndOffLineForm newAndOffLineForm) {
        String farmID = newAndOffLineForm.getFarmID();
        if (StringUtils.isNotEmpty(farmID)) {
            EartagDataServiceImpl.checkFarmCode(farmID, baseFarmInfoDao);
        }
        IPage<NewAndOffLineEartagEntity> iPage = new Page<>(newAndOffLineForm.getPageNo(), newAndOffLineForm.getPageSize());
        return eartagDataDao.selectChangeEartagList(iPage, newAndOffLineForm.getStartDate(), newAndOffLineForm.getEndDate(), newAndOffLineForm.getFarmID(), newAndOffLineForm.getStage());
    }

    private List<String> buildList(String first, String second) {
        List<String> list = new ArrayList<>();
        if (StringUtils.isNotEmpty(first)) {
            list.addAll(Arrays.asList(first.split(",")));
        }
        if (StringUtils.isNotEmpty(second)) {
            list.removeAll(Arrays.asList(second.split(",")));
        }
        return list;
    }


    private void buildPage(IPage source, IPage target) {
        target.setPages(source.getPages());
        target.setTotal(source.getTotal());
        target.setCurrent(source.getCurrent());
        target.setSize(source.getSize());
    }
}
